package android.project.Koo;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;

/**
 * Gestisce la creazione e l'upgrade del DB e mette a disposizione
 * metodi utili per la gestione dei dati presenti.
 * 
 * @author Francesco Marcantoni, Francesco Rigo, Michele Ambrosi 
 */
public class DBAdapter
{
    private static final String TAG = "DBAdapter";

    /** Nome del DB */
    private static final String DATABASE_NAME = "koodb";
    /** Versione del DB, è usata per le operazione di upgrade */
    private static final int DATABASE_VERSION = 5;

    /** Nome della tabella dei contatti */
    private static final String DATABASE_CONTACTS_TABLE = "contacts";
    
    /* Identificatori per le colonne della tabella dei contatti */
    public static final String KEY_CONTACTS_ROWID = "_id";
    public static final String KEY_CONTACTS_NAME = "name";
    public static final String KEY_CONTACTS_PHONE = "number";
    public static final String KEY_CONTACTS_EMAIL = "email";
    public static final String KEY_CONTACTS_COMPANY = "company";
    
    /* Posizioni delle colonne della tabella dei contatti */
    public static final int POS_CONTACTS_ROWID = 0;
    public static final int POS_CONTACTS_NAME = 1;
    public static final int POS_CONTACTS_PHONE = 2;
    public static final int POS_CONTACTS_EMAIL = 3;
    public static final int POS_CONTACTS_COMPANY = 4;
    
    /** Query utilizzata per la creazione del DB */
    private static final String QUERY_DATABASE_CREATE =
        "CREATE TABLE " + DATABASE_CONTACTS_TABLE + "("
        + KEY_CONTACTS_ROWID + " integer primary key autoincrement,"
        + KEY_CONTACTS_NAME + " text not null,"
        + KEY_CONTACTS_PHONE + " text not null,"
        + KEY_CONTACTS_EMAIL + " text not null,"
        + KEY_CONTACTS_COMPANY + " text not null"
        + ");";

    /** Android context */
    private final Context context;
    
    /** DataBase helper */ 
    private DatabaseHelper DBHelper;
    
    /** DataBase reference */
    private SQLiteDatabase db;

    /**
     * Costruttore.
     * 
     * @param ctx Android context
     */
    public DBAdapter(Context ctx) 
    {
        this.context = ctx;
        DBHelper = new DatabaseHelper(context);
    }

    /**
     * Gestisce la creazione e l'upgrade del DB.
     * 
     * @author Francesco Marcantoni, Francesco Rigo, Michele Ambrosi
     */
    private static class DatabaseHelper extends SQLiteOpenHelper 
    {
    	/**
    	 * Costruttore.
    	 * 
    	 * @param context Android context
    	 */
        DatabaseHelper(Context context)
        {
            super(context, DATABASE_NAME, null, DATABASE_VERSION);
        }

        @Override
        /**
         * Metodo invocato dal sistema Android dopo la creazione del DB.
         * 
         * @param db DataBase reference
         */
        public void onCreate(SQLiteDatabase db) 
        {
        	Log.w(TAG, "Create database");
            db.execSQL(QUERY_DATABASE_CREATE);
        	Log.w(TAG, "Database created");
        }

        @Override
        /**
         * Metodo invocato dal sistema Android durante l'upgrade del DB.
         * 
         * @param db DataBase reference
         * @param oldVersion Vecchio numero di versione del DB
         * @param newVersion Nuovo numero di versione del DB
         */
        public void onUpgrade(SQLiteDatabase db,
        		int oldVersion,
        		int newVersion) 
        {
            Log.w(TAG, "Upgrading database from version " + oldVersion 
                  + " to "
                  + newVersion + ", which will destroy all old data");
            // Elimino tutte le tabelle
            db.execSQL("DROP TABLE IF EXISTS " + DATABASE_CONTACTS_TABLE);
            onCreate(db);
        }
    }

    /**
     * Apre il DB in scrittura e lettura; questo metodo deve essere invocato prima di utilizzare
     * il DB.
     */
    public DBAdapter open() throws SQLException 
    {
        db = DBHelper.getWritableDatabase();
        return this;
    }

    /**
     * Chiude il DB.
     */
    public void close() 
    {
        DBHelper.close();
    }
    
    /**
     * Inserisce un contatto all'interno del DB.
     * 
     * @param contact Contatto da aggiungere
     * @return Identificatore di riga
     */
    public long insertContact(Contact contact) 
    {
    	// Creo una nuova riga vuota per la tabella dei contatti
        ContentValues initialValues = new ContentValues();
        
        // Inserisco i dati all'interno delle rispettive colonne
        initialValues.put(KEY_CONTACTS_NAME, contact.getName());
        initialValues.put(KEY_CONTACTS_PHONE, contact.getPhone());
        initialValues.put(KEY_CONTACTS_EMAIL, contact.getEMail());
        initialValues.put(KEY_CONTACTS_COMPANY, contact.getCompany());
        
        // Inserisco la riga all'interno della tabella dei contatti
        long id = db.insert(DATABASE_CONTACTS_TABLE, null, initialValues);
        // Salvo l'identificatore di riga all'interno del corrispondente
        // campo dell'oggetto Contact
        contact.setId(id);
        return id;
    }

    /**
     * Elimina il contatto specificato dal DB.
     * 
     * @param rowId Identificatore di riga del contatto
     * @return
     */
    public boolean deleteContact(long rowId) 
    {
        return db.delete(DATABASE_CONTACTS_TABLE, KEY_CONTACTS_ROWID + "=" + rowId, null) > 0;
    }

    /**
     * Restituisce tutti i contatti presenti all'interno del DB.
     * Esattamente come per il risultato di una query SQL, viene
     * restituito un riferimento (un cursore) alla tabella contenente
     * i dati. 
     * 
     * @return Un oggetto di tipo Cursor per la tabella specificata
     */
    public Cursor getAllContacts()
    {
        return db.query(false, DATABASE_CONTACTS_TABLE, new String[] {
        		KEY_CONTACTS_ROWID,
        		KEY_CONTACTS_NAME,
        		KEY_CONTACTS_PHONE,
        		KEY_CONTACTS_EMAIL,
        		KEY_CONTACTS_COMPANY},
                null,
                null,
                null,
                null,
                null,
                null);
    }
    
    /**
     * Elimina tutti i contatti presenti all'interno del DB.
     * 
     * @return true se viene eliminato almeno un contatto, false altrimenti
     */
    public boolean deleteAllContacts() 
    {
        return db.delete(DATABASE_CONTACTS_TABLE,
        		null, null) > 0;
    }
    
    /**
     * Restituisce un contatto specifico; Viene fornito un cursore
     * ad una tabella di una riga, identica alla tabella dei contatti
     * presente nel DB, contenente i dati del contatto specificato.
     *  
     * @param rowId Identificatore di riga del contatto
     * @return Un oggetto di tipo Cursor per la tabella specificata
     * @throws SQLException
     */
    public Cursor getContact(long rowId) throws SQLException 
    {
        Cursor mCursor =
                db.query(true, DATABASE_CONTACTS_TABLE, new String[] {
                        KEY_CONTACTS_ROWID,
                		KEY_CONTACTS_NAME,
                		KEY_CONTACTS_PHONE,
                		KEY_CONTACTS_EMAIL,
                		KEY_CONTACTS_COMPANY},
                		KEY_CONTACTS_ROWID + "=" + rowId, 
                		null,
                		null, 
                		null, 
                		null, 
                		null);
        
        if (mCursor != null)
            mCursor.moveToFirst();

        return mCursor;
    }

    /**
     * Aggiorna le informazioni di un contatto.
     * 
     * @param rowId Identificatore di riga per il contatto
     * @param contact Nuove informazioni del contatto
     * @return true se viene aggiornato almeno un contatto, false altrimenti
     */
    public boolean updateContact(long rowId, Contact contact) 
    {
        ContentValues args = new ContentValues();
        args.put(KEY_CONTACTS_NAME, contact.getName());
        args.put(KEY_CONTACTS_PHONE, contact.getPhone());
        args.put(KEY_CONTACTS_EMAIL, contact.getEMail());
        args.put(KEY_CONTACTS_COMPANY, contact.getCompany());
        
        return db.update(DATABASE_CONTACTS_TABLE, args, 
                         KEY_CONTACTS_ROWID + "=" + rowId, null) > 0;
    }
}
